#'Merge Observations To Experiments 
#'
#'Some indicators are defined for specific temporal periods (e.g.: summer from June 21st to September 21st). If the initialization forecast date is later than the one required for the indicator (e.g.: July 1st), the user may want to merge past observations, or other reference, to the forecast to compute the indicator. The function \code{MergeObs2Exp} takes care of this steps.
#'
#'@param ref a multidimensional array with named dimensions.
#'@param ref_dates a vector of dates or a multidimensional array of dates with named dimensions matching the dimensions on parameter 'ref'.
#'@param data a multidimensional array with named dimensions.
#'@param data_dates a vector of dates or a multidimensional array of dates with named dimensions matching the dimensions on parameter 'data'.
#'@param start an optional parameter to defined the initial date of the period to select from the data by providing a list of two elements: the initial date of the period and the initial month of the period. By default it is set to NULL and the indicator is computed using all the data provided in \code{data}.
#'@param end an optional parameter to defined the final date of the period to select from the data by providing a list of two elements: the final day of the period and the final month of the period. By default it is set to NULL and the indicator is computed using all the data provided in \code{data}.
#'@param time_dim a character string indicating the name of the temporal dimension. By default, it is set to 'ftime'. More than one dimension name matching the dimensions provided in the object \code{data$data} can be specified. This dimension is required to subset the data in a requested period.
#'@param memb_dim a character string indicating the name of the dimension in which the ensemble members are stored.
#'@param sdate_dim a character string indicating the name of the dimension in which the initialization dates are stored. 
#'@param ncores an integer indicating the number of cores to use in parallel computation.
#'
#'@return A multidimensional array with named dimensions.
#'
#'@import multiApply
MergeRefToData <- function(ref, ref_dates, data, data_dates, start, end,
                           time_dim = 'time', sdate_dim = 'sdate',
                           memb_dim = 'member', ncores = NULL) {
  if (!is.array(ref)) {
    dim(ref) <- c(length(ref))
    names(dim(ref)) <- time_dim
  }
  if (!is.array(data)) {
    dim(data) <- c(length(data))
    names(dim(data)) <- time_dim
  }
  if (is.null(dim(ref_dates))) {
    dim(ref_dates) <- length(ref_dates)
    names(dim(ref_dates)) <- time_dim
  }
  if (is.null(dim(data_dates))) {
    dim(data_dates) <- length(data_dates)
    names(dim(data_dates)) <- time_dim
  }
  ini <- ClimProjDiags::Subset(data_dates, along = time_dim, indices = 1)
  day <- format(ini, "%d")
  month <- format(ini, "%m")
  if (length(day) > 1) {
    if (any(day[1] != day)) {
      warning("Different initialization days found in 'data_dates'.",
              " Only the first one is being used. Check the result.")
    }
    day <- day[1]
   }
   if (length(month) > 1) {
     if (any(month[1] != month)) {
       warning("Different initialization months found in 'data_dates'.", 
               " Only the first one is being used. Check the result.")
     }
     month <- month[1]
   }             
 
  #check_refd <- SelectPeriodOnDates(ref_dates, start = start,
  #                          end = list(day, month),
  #                          time_dim = time_dim)
  ref <- SelectPeriodOnData(ref, dates = ref_dates, start = start,
                         end = list(day, month), 
                         time_dim = time_dim)
  if ((sdate_dim %in% names(dim(data))) && dim(data)[sdate_dim] > 1 && 
      !sdate_dim %in% names(dim(ref))) {
    dim(ref) <- c(length(ref)/dim(data)[sdate_dim], dim(data)[sdate_dim])
    names(dim(ref)) <- c(time_dim, sdate_dim)
  }
  if (length(dim(data)) != length(dim(ref))) {
    dif_dims <- which(names(dim(data)) %in% names(dim(ref)) == FALSE)
    for (i in dif_dims) {
      ref <- s2dv::InsertDim(ref, posdim = i, lendim = dim(data)[i], 
                             name = names(dim(data))[i])
    }
  } 
  #check_datad <- SelectPeriodOnDates(data_dates,
  #                          start = list(day, month),
  #                          end = end,
  #                          time_dim = time_dim)

  data <- SelectPeriodOnData(data, dates = data_dates, start = list(day, month),
                               end = end, time_dim = time_dim, ncores = 1)
  data <- abind::abind(ref, data, along = which(names(dim(data)) == time_dim))
  return(data)
}
   

